@@ -7,7 +7,11 @@ def parse_bandwidth_metrics(powermetrics_parse):
     bandwidth_metrics_dict = {}
     data_fields = ["PCPU0 DCS RD", "PCPU0 DCS WR",
                    "PCPU1 DCS RD", "PCPU1 DCS WR",
+                   "PCPU2 DCS RD", "PCPU2 DCS WR",
+                   "PCPU3 DCS RD", "PCPU3 DCS WR",
                    "PCPU DCS RD", "PCPU DCS WR",
+                   "ECPU0 DCS RD", "ECPU0 DCS WR",
+                   "ECPU1 DCS RD", "ECPU1 DCS WR",
                    "ECPU DCS RD", "ECPU DCS WR",
                    "GFX DCS RD", "GFX DCS WR",
                    "ISP DCS RD", "ISP DCS WR",
@@ -16,9 +20,13 @@ def parse_bandwidth_metrics(powermetrics_parse):
                    "VDEC DCS RD", "VDEC DCS WR",
                    "VENC0 DCS RD", "VENC0 DCS WR",
                    "VENC1 DCS RD", "VENC1 DCS WR",
+                   "VENC2 DCS RD", "VENC2 DCS WR",
+                   "VENC3 DCS RD", "VENC3 DCS WR",
                    "VENC DCS RD", "VENC DCS WR",
                    "JPG0 DCS RD", "JPG0 DCS WR",
                    "JPG1 DCS RD", "JPG1 DCS WR",
+                   "JPG2 DCS RD", "JPG2 DCS WR",
+                   "JPG3 DCS RD", "JPG3 DCS WR",
                    "JPG DCS RD", "JPG DCS WR",
                    "DCS RD", "DCS WR"]
     for h in data_fields:
@@ -28,22 +36,34 @@ def parse_bandwidth_metrics(powermetrics_parse):
             bandwidth_metrics_dict[l["name"]] = l["value"]/(1e9)
     bandwidth_metrics_dict["PCPU DCS RD"] = bandwidth_metrics_dict["PCPU DCS RD"] + \
         bandwidth_metrics_dict["PCPU0 DCS RD"] + \
-        bandwidth_metrics_dict["PCPU1 DCS RD"]
+        bandwidth_metrics_dict["PCPU1 DCS RD"] + \
+        bandwidth_metrics_dict["PCPU2 DCS RD"] + \
+        bandwidth_metrics_dict["PCPU3 DCS RD"]
     bandwidth_metrics_dict["PCPU DCS WR"] = bandwidth_metrics_dict["PCPU DCS WR"] + \
         bandwidth_metrics_dict["PCPU0 DCS WR"] + \
-        bandwidth_metrics_dict["PCPU1 DCS WR"]
+        bandwidth_metrics_dict["PCPU1 DCS WR"] + \
+        bandwidth_metrics_dict["PCPU2 DCS WR"] + \
+        bandwidth_metrics_dict["PCPU3 DCS WR"]
     bandwidth_metrics_dict["JPG DCS RD"] = bandwidth_metrics_dict["JPG DCS RD"] + \
         bandwidth_metrics_dict["JPG0 DCS RD"] + \
-        bandwidth_metrics_dict["JPG1 DCS RD"]
+        bandwidth_metrics_dict["JPG1 DCS RD"] + \
+        bandwidth_metrics_dict["JPG2 DCS RD"] + \
+        bandwidth_metrics_dict["JPG3 DCS RD"]
     bandwidth_metrics_dict["JPG DCS WR"] = bandwidth_metrics_dict["JPG DCS WR"] + \
         bandwidth_metrics_dict["JPG0 DCS WR"] + \
-        bandwidth_metrics_dict["JPG1 DCS WR"]
+        bandwidth_metrics_dict["JPG1 DCS WR"] + \
+        bandwidth_metrics_dict["JPG2 DCS WR"] + \
+        bandwidth_metrics_dict["JPG3 DCS WR"]
     bandwidth_metrics_dict["VENC DCS RD"] = bandwidth_metrics_dict["VENC DCS RD"] + \
         bandwidth_metrics_dict["VENC0 DCS RD"] + \
-        bandwidth_metrics_dict["VENC1 DCS RD"]
+        bandwidth_metrics_dict["VENC1 DCS RD"] + \
+        bandwidth_metrics_dict["VENC2 DCS RD"] + \
+        bandwidth_metrics_dict["VENC3 DCS RD"]
     bandwidth_metrics_dict["VENC DCS WR"] = bandwidth_metrics_dict["VENC DCS WR"] + \
         bandwidth_metrics_dict["VENC0 DCS WR"] + \
-        bandwidth_metrics_dict["VENC1 DCS WR"]
+        bandwidth_metrics_dict["VENC1 DCS WR"] + \
+        bandwidth_metrics_dict["VENC2 DCS WR"] + \
+        bandwidth_metrics_dict["VENC3 DCS WR"]
     bandwidth_metrics_dict["MEDIA DCS"] = sum([
         bandwidth_metrics_dict["ISP DCS RD"], bandwidth_metrics_dict["ISP DCS WR"],
         bandwidth_metrics_dict["STRM CODEC DCS RD"], bandwidth_metrics_dict["STRM CODEC DCS WR"],
@@ -64,12 +84,34 @@ def parse_cpu_metrics(powermetrics_parse):
         name = cluster["name"]
         cpu_metric_dict[name+"_freq_Mhz"] = int(cluster["freq_hz"]/(1e6))
         cpu_metric_dict[name+"_active"] = int((1 - cluster["idle_ratio"])*100)
+    if "E-Cluster_active" not in cpu_metric_dict:
+        # M1 Ultra
+        cpu_metric_dict["E-Cluster_active"] = int(
+            (cpu_metric_dict["E0-Cluster_active"] + cpu_metric_dict["E1-Cluster_active"])/2)
+    if "E-Cluster_freq_Mhz" not in cpu_metric_dict:
+        # M1 Ultra
+        cpu_metric_dict["E-Cluster_freq_Mhz"] = max(
+            cpu_metric_dict["E0-Cluster_freq_Mhz"], cpu_metric_dict["E1-Cluster_freq_Mhz"])
     if "P-Cluster_active" not in cpu_metric_dict:
-        cpu_metric_dict["P-Cluster_active"] = int((
-            cpu_metric_dict["P0-Cluster_active"] + cpu_metric_dict["P1-Cluster_active"])/2)
+        if "P2-Cluster_active" in cpu_metric_dict:
+            # M1 Ultra
+            cpu_metric_dict["P-Cluster_active"] = int((cpu_metric_dict["P0-Cluster_active"] + cpu_metric_dict["P1-Cluster_active"] +
+                                                      cpu_metric_dict["P2-Cluster_active"] + cpu_metric_dict["P3-Cluster_active"]) / 4)
+        else:
+            cpu_metric_dict["P-Cluster_active"] = int(
+                (cpu_metric_dict["P0-Cluster_active"] + cpu_metric_dict["P1-Cluster_active"])/2)
     if "P-Cluster_freq_Mhz" not in cpu_metric_dict:
-        cpu_metric_dict["P-Cluster_freq_Mhz"] = max(
-            cpu_metric_dict["P0-Cluster_freq_Mhz"], cpu_metric_dict["P1-Cluster_freq_Mhz"])
+        if "P2-Cluster_freq_Mhz" in cpu_metric_dict:
+            # M1 Ultra
+            freqs = [
+                cpu_metric_dict["P0-Cluster_freq_Mhz"],
+                cpu_metric_dict["P1-Cluster_freq_Mhz"],
+                cpu_metric_dict["P2-Cluster_freq_Mhz"],
+                cpu_metric_dict["P3-Cluster_freq_Mhz"]]
+            cpu_metric_dict["P-Cluster_freq_Mhz"] = max(freqs)
+        else:
+            cpu_metric_dict["P-Cluster_freq_Mhz"] = max(
+                cpu_metric_dict["P0-Cluster_freq_Mhz"], cpu_metric_dict["P1-Cluster_freq_Mhz"])
     # power
     cpu_metric_dict["ane_W"] = cpu_metrics["ane_energy"]/1000
     cpu_metric_dict["dram_W"] = cpu_metrics["dram_energy"]/1000
